==> mp3plot-0.5.1.tar.gz: mp3plot-0.5.1/src/plotters/colourschemegen.pl <==
#!/usr/bin/env perl

# Copyright (c) 2009, Toni Corvera. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# $Id: colourschemegen.pl 1700 2009-04-24 19:06:20Z  $
#
# This script generates a C++ source file with the colour scheme initialisation
# It takes an HSV colour and produces a scale from that colour towards more
# "pure" colours. It outputs a C++ code file and an XPM image file with a
# sample of the scheme.
# The scale is produced by augmenting the Saturation component of the HSV
# colour. This easy method (to do programatically) produces colours on a
# "scale" from darker to lighter.
# Additionally the hue can also be modified in a limited fashion: it changes
# which primary colour influences the colour: In the HSV system, all colours
# are placed in a cylinder, each primary colour dominates a 60º arc, while
# the remaning 3 60º arcs are "secondary" colours (primary=RGB, secondary=CMY)
# The second argument to this script defined how many zones of influence
# should be used: if 1 to 3, the hue will be rotated in 120 degree steps,
# (e.g. Red to Blue to Green), if 4 to 6 the hue will be rotated in 60 degree
# steps (e.g. Red to Magenta to Blue to Cyan to Green to Yellow)

# INSTRUCTIONS:
# A colourscheme.inc.cc is provided in the distribution tarball, to
# (re-)generate it, switch to the src/plotters/ subdirectory and type
# $ make colourscheme.inc.cc
#
# To change the colourscheme pass a different set of values, e.g.
# $ colourschemegen.pl 000030 1 R
# or
# $ make colourscheme.inc.cc CSARGS='000030 1 R'

# Note this is the first time I write perl code, sorry for the bloody mess :)

# TODO:
# When using more than one hue, colours should be sorted by value otherwise,
# e.g. light green is followed by dark red

use Graphics::ColorObject;
use File::Basename;

use strict;
#use warnings;

my $basename=basename($0);

sub round {
	my($number) = shift;
	return int($number + .5 * ($number <=> 0));
}

sub get {
	my($h) = shift;
	my($s) = shift;
	my($v) = shift;
	my ($r, $g, $b);

	($r, $g, $b) = @{ Graphics::ColorObject->new_HSV([$h, $s, $v])->as_RGB() };

	$r= round($r);
	$g= round($g);
	$b= round($b);

	return ($r, $g, $b);
}

my $BAD_BR_COL=128;

# This is the C++ code template
my $template = <<EOF;
%s
// \$Id\$
/*
 * File automatically generated by the $basename script, part of mp3plot.
 * This file is meant to be '#included' by image_plotter.cc (although it should
 * also compile by itself).
 * See the $basename script for details on how to (re-)generate this file.
 *
 * Command-line used to generate this file: $basename @ARGV
 */
#include "../common.h"
#include "../mp3/bitrate.h"
#include "colour.h"

#include <cassert>
#include <vector>
#include <map>

#include <iostream>

namespace mp3plot {
namespace impl {
namespace colourscheme {

using namespace std;
using namespace net_outlyer;

const map<bitrate_t, rgb_type>& get_map() {
	static map<bitrate_t, rgb_type> m;

	if (m.empty()) {
		typedef colour_type<int, 255> ict;
		bitrate_t idx;
		const vector<bitrate_t> & brs = mp3::standard_bitrates::as_vector();
		vector<bitrate_t>::const_iterator it = brs.begin();
		// Free-form
		%s

		// Standard bitrates
%s
		// Bad bitrate
		%s
	}

	return m;
}

} // ns colourscheme
} // ns impl
} // ns mp3plot

EOF

if (@ARGV < 2) {
	print "Usage: $basename <RGB Hex Colour> <Number of hues (1-6, default=1)> [Direction (R or L, default R)]\n";
	exit 1;
}

my $hexc = shift;
my $numHues = shift;
my $direction = shift;

if (length($hexc) != 7) {
	if (length($hexc) == 6) {
		$hexc = "#$hexc";
	}
	else {
		print STDERR "Error: RGB colour must be in the HEX format (e.g. '#aabb90' or 'aabb90')\n";
		exit 3;
	}
}

if ($numHues < 1 || $numHues > 6) {
	print STDERR "Error: The number of hues must be 1..6\n";
	exit 4;
}

if (!$direction) {
	$direction = "R";
}
elsif ($direction != "R" && $direction != "L") {
	print STDERR "Error: Direction must be R or L\n";
	exit 7;
}

my $R = hex substr $hexc,1,2;
my $G = hex substr $hexc,3,2;
my $B = hex substr $hexc,5,2;

my ($H, $S, $V);
# Get the HSV representation
($H, $S, $V) = @{ Graphics::ColorObject->new_RGB([$R, $G, $B])->as_HSV() };
# ColorObject produces HSV with ranges H 0..360, S 0..1f, V 0..255
printf STDERR "Input: RGB=$hexc ($R,$G,$B)\tHSV [%d,%.2f,%d]\n", $H, $S, $V;

my $partitions = shift;
my $sectors = 2;

my $arc;
my @hues;
if ($numHues == 1) {
	push @hues,$H;
}
elsif ($numHues <= 6) {
	$arc = ($numHues < 4) ? 120.0 : 60.0;
	my $nextHue = round($H);
	for (my $i=0; $i<$numHues; ++$i) {
		push @hues,$nextHue;
		if ("R" eq $direction) {
			$nextHue = ( $nextHue + $arc ) % 360.0;
		}
		else { # $direction eq "L"
			$nextHue = ( ( $nextHue - $arc ) + 360.0 ) % 360.0;
		}
	}
}

# C++ code to fill the "m" STL map, see the heredoc at the end of this file
# for the rest of the C++ code
my $cxx_init_code = "";

my $colidx = 65; # XPM colour palette index
my $xpm = "";    # XPM image pixels
my $palette = ""; # XPM palette contents

print STDERR "$numHues hues (@hues), arc=$arc, direction=$direction\n";

my ($r, $g, $b, $h, $s, $v);

#############################
#
# Bitrate -> Colour mapping
#
##############################

my ($bH, $bS, $bV);
# Bad bitrate
($bH,$bS,$bV) =  @{ Graphics::ColorObject->new_RGB([$BAD_BR_COL,$BAD_BR_COL,$BAD_BR_COL])->as_HSV() };
printf(STDERR "[ b] HSV: %d,%4.2f,%6.2f   <-->   RGB: $BAD_BR_COL,$BAD_BR_COL,$BAD_BR_COL\n", $bH, $bS, $bV);
$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), $BAD_BR_COL, $BAD_BR_COL, $BAD_BR_COL );
my $cxx_bad_bitrate = "m.insert( make_pair(mp3::standard_bitrates::bad_bitrate(), ".
                                               "ict($BAD_BR_COL).convert<double,1>()) );";

# Free-form (bitrate 0), it is a special case, will be handled differently
# at some point in the future
# Automatic generation:
#my $fH = $H - 120;
#my $fS = $S - (1.0 - $S) / 2;
#my $fV = $V + (255 - $V) / 2;
#($r, $g, $b) = get($fH, $fS, $fV); # Rotate to the previous primary colour (B -> G -> R -> B)
#printf(STDERR "[ f] HSV: %d,%4.2f,%6.2f   <-->   RGB: $r,$g,$b\n", $fH, $fS, $fV);
#$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), $r, $g, $b );
#my $cxx_free_form = "m.insert( make_pair(*it++, ict($r,$g,$b).convert<double,1>()) );";
my ($fH, $fS, $fV) = @{ Graphics::ColorObject->new_RGB([255,255,255])->as_HSV() };
printf(STDERR "[ f] HSV: %d,%4.2f,%6.2f   <-->   RGB: 255,255,255\n", $fH, $fS, $fV);
$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), 255,255,255 );
my $cxx_free_form = "m.insert( make_pair(*it++, rgb_type(1)) );";

my @hsv_values = [];
my $valstep = (255 - $V) / ((14-1) / $numHues);
my $idx = 0;
huefill:
foreach my $hue (@hues) {
	$v = $V;
	while ($v < 256) {
		$hsv_values[$idx++] = [ $hue, $S, $v ];
		$v += $valstep;

		if ($idx == 14) {
			last huefill;
		}
	}
}

my $idx = 1;
foreach my $value (@hsv_values) {
	($h, $s, $v) = @{$value};
	($r, $g, $b) = get($h, $s, $v);
	printf(STDERR "[%2d] HSV: %d,%4.2f,%6.2f   <-->   RGB: $r,$g,$b\n", $idx++, $h, $s, $v);
	
	$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), $r, $g, $b );
	$cxx_init_code .= "\t\tidx = *it++;\n".
	                  "\t\tassert( it != brs.end() );\n".
	                  "\t\tm.insert( make_pair(idx, ict($r,$g,$b).convert<double,1>()) );\n";
}

#############################
#
# XPM sample of the scheme
#
############################

my $COLORWIDTH=100;
my $numcolors = $colidx - 65;
my $W = $COLORWIDTH * $numcolors;

open (FILE, '>cscheme_sample.xpm');
print FILE "! XPM2\n" ;
print FILE "$W 128 $numcolors 1\n";
print(FILE $palette);
for (my $i=0; $i<128; ++$i) {
	for (my $col=65; $col<$colidx; ++$col) {
		my $sym = chr($col);
		for (my $n=0; $n<$COLORWIDTH; ++$n) {
			$xpm .= $sym;
		}
	}
	$xpm .="\n";
}
print(FILE $xpm);
close (FILE);

#############################
#
# C++ code file
#
#############################

# Copy the license from old_colourscheme.inc.cc...
open(IN, 'old_colourscheme.inc.cc');
my $license = "";
for (my $i=0; $i<19; ++$i) {
	$license .= <IN>;
}
close(IN);

open(CS, '>colourscheme.inc.cc');
printf CS $template,$license,$cxx_free_form,$cxx_init_code,$cxx_bad_bitrate;
close(CS);

==> mp3plot-0.6.0.tar.gz: mp3plot-0.6.0/src/plotters/colourschemegen.pl <==
#!/usr/bin/env perl

# Copyright (c) 2009, Toni Corvera. All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
# FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
# OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# $Id: colourschemegen.pl 1344 2009-06-22 01:11:09Z toni $
#
# This script generates a C++ source file with the colour scheme initialisation
# It takes an HSV colour and produces a scale from that colour towards more
# "pure" colours. It outputs a C++ code file and an XPM image file with a
# sample of the scheme.
# The scale is produced by augmenting the Saturation component of the HSV
# colour. This easy method (to do programatically) produces colours on a
# "scale" from darker to lighter.
# Additionally the hue can also be modified in a limited fashion: it changes
# which primary colour influences the colour: In the HSV system, all colours
# are placed in a cylinder, each primary colour dominates a 60º arc, while
# the remaning 3 60º arcs are "secondary" colours (primary=RGB, secondary=CMY)
# The second argument to this script defined how many zones of influence
# should be used: if 1 to 3, the hue will be rotated in 120 degree steps,
# (e.g. Red to Blue to Green), if 4 to 6 the hue will be rotated in 60 degree
# steps (e.g. Red to Magenta to Blue to Cyan to Green to Yellow)

# INSTRUCTIONS:
# A colourscheme.inc.cc is provided in the distribution tarball, to
# (re-)generate it, switch to the src/plotters/ subdirectory and type
# $ make colourscheme.inc.cc
#
# To change the colourscheme pass a different set of values, e.g.
# $ colourschemegen.pl 000030 1 R
# or
# $ make colourscheme.inc.cc CSARGS='000030 1 R'

# Note this is the first time I write perl code, sorry for the bloody mess :)

# TODO:
# When using more than one hue, colours should be sorted by value otherwise,
# e.g. light green is followed by dark red

use Graphics::ColorObject;
use File::Basename;

use strict;
#use warnings;

my $basename=basename($0);

sub round {
	my($number) = shift;
	return int($number + .5 * ($number <=> 0));
}

sub get {
	my($h) = shift;
	my($s) = shift;
	my($v) = shift;
	my ($r, $g, $b);

	($r, $g, $b) = @{ Graphics::ColorObject->new_HSV([$h, $s, $v])->as_RGB() };

	$r= round($r);
	$g= round($g);
	$b= round($b);

	return ($r, $g, $b);
}

my $BAD_BR_COL=128;

# This is the C++ code template
my $template = <<EOF;
%s
// \$Id\$
/*
 * File automatically generated by the $basename script, part of mp3plot.
 * This file is meant to be '#included' by image_plotter.cc (although it should
 * also compile by itself).
 * See the $basename script for details on how to (re-)generate this file.
 *
 * Command-line used to generate this file: $basename @ARGV
 */
#include "../common.h"
#include "../mp3/bitrate.h"
#include "colour.h"

#include <cassert>
#include <vector>
#include <map>

#include <iostream>

namespace mp3plot {
namespace impl {
namespace colourscheme {

using namespace std;
using namespace net_outlyer;

const map<uint8_t, rgb_type>& get_map() {
	static map<uint8_t, rgb_type> m;

	if (m.empty()) {
		typedef colour_type<int, 255> ict;
		uint8_t header_idx = 0;
		// Free-form
		%s

		// Standard bitrates
%s
		// Bad bitrate
		%s
	}

	return m;
}

} // ns colourscheme
} // ns impl
} // ns mp3plot

EOF

if (@ARGV < 2) {
	print "Usage: $basename <RGB Hex Colour> <Number of hues (1-6, default=1)> [Direction (R or L, default R)]\n";
	exit 1;
}

my $hexc = shift;
my $numHues = shift;
my $direction = shift;

if (length($hexc) != 7) {
	if (length($hexc) == 6) {
		$hexc = "#$hexc";
	}
	else {
		print STDERR "Error: RGB colour must be in the HEX format (e.g. '#aabb90' or 'aabb90')\n";
		exit 3;
	}
}

if ($numHues < 1 || $numHues > 6) {
	print STDERR "Error: The number of hues must be 1..6\n";
	exit 4;
}

if (!$direction) {
	$direction = "R";
}
elsif ($direction != "R" && $direction != "L") {
	print STDERR "Error: Direction must be R or L\n";
	exit 7;
}

my $R = hex substr $hexc,1,2;
my $G = hex substr $hexc,3,2;
my $B = hex substr $hexc,5,2;

my ($H, $S, $V);
# Get the HSV representation
($H, $S, $V) = @{ Graphics::ColorObject->new_RGB([$R, $G, $B])->as_HSV() };
# ColorObject produces HSV with ranges H 0..360, S 0..1f, V 0..255
printf STDERR "Input: RGB=$hexc ($R,$G,$B)\tHSV [%d,%.2f,%d]\n", $H, $S, $V;

my $partitions = shift;
my $sectors = 2;

my $arc;
my @hues;
if ($numHues == 1) {
	push @hues,$H;
}
elsif ($numHues <= 6) {
	$arc = ($numHues < 4) ? 120.0 : 60.0;
	my $nextHue = round($H);
	for (my $i=0; $i<$numHues; ++$i) {
		push @hues,$nextHue;
		if ("R" eq $direction) {
			$nextHue = ( $nextHue + $arc ) % 360.0;
		}
		else { # $direction eq "L"
			$nextHue = ( ( $nextHue - $arc ) + 360.0 ) % 360.0;
		}
	}
}

# C++ code to fill the "m" STL map, see the heredoc at the end of this file
# for the rest of the C++ code
my $cxx_init_code = "";

my $colidx = 65; # XPM colour palette index
my $xpm = "";    # XPM image pixels
my $palette = ""; # XPM palette contents

print STDERR "$numHues hues (@hues), arc=$arc, direction=$direction\n";

my ($r, $g, $b, $h, $s, $v);

#############################
#
# Bitrate -> Colour mapping
#
##############################

my ($bH, $bS, $bV);
# Bad bitrate
($bH,$bS,$bV) =  @{ Graphics::ColorObject->new_RGB([$BAD_BR_COL,$BAD_BR_COL,$BAD_BR_COL])->as_HSV() };
printf(STDERR "[ b] HSV: %d,%4.2f,%6.2f   <-->   RGB: $BAD_BR_COL,$BAD_BR_COL,$BAD_BR_COL\n", $bH, $bS, $bV);
$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), $BAD_BR_COL, $BAD_BR_COL, $BAD_BR_COL );
my $cxx_bad_bitrate =  "assert( m.end() == m.find(0x0F) );\n".
                       "\t\tm.insert( make_pair(0x0F, ".
                                               "ict($BAD_BR_COL).convert<double,1>()) );";

# Free-form (bitrate 0), it is a special case, will be handled differently
# at some point in the future
# Automatic generation:
#my $fH = $H - 120;
#my $fS = $S - (1.0 - $S) / 2;
#my $fV = $V + (255 - $V) / 2;
#($r, $g, $b) = get($fH, $fS, $fV); # Rotate to the previous primary colour (B -> G -> R -> B)
#printf(STDERR "[ f] HSV: %d,%4.2f,%6.2f   <-->   RGB: $r,$g,$b\n", $fH, $fS, $fV);
#$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), $r, $g, $b );
#my $cxx_free_form = "m.insert( make_pair(*it++, ict($r,$g,$b).convert<double,1>()) );";
my ($fH, $fS, $fV) = @{ Graphics::ColorObject->new_RGB([255,255,255])->as_HSV() };
printf(STDERR "[ f] HSV: %d,%4.2f,%6.2f   <-->   RGB: 255,255,255\n", $fH, $fS, $fV);
$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), 255,255,255 );
my $cxx_free_form = "m.insert( make_pair(0, rgb_type(1)) );";

my @hsv_values = [];
my $valstep = (255 - $V) / ((14-1) / $numHues);
my $idx = 0;
huefill:
foreach my $hue (@hues) {
	$v = $V;
	while ($v < 256) {
		$hsv_values[$idx++] = [ $hue, $S, $v ];
		$v += $valstep;

		if ($idx == 14) {
			last huefill;
		}
	}
}

my $idx = 1;
foreach my $value (@hsv_values) {
	($h, $s, $v) = @{$value};
	($r, $g, $b) = get($h, $s, $v);
	printf(STDERR "[%2d] HSV: %d,%4.2f,%6.2f   <-->   RGB: $r,$g,$b\n", $idx++, $h, $s, $v);
	
	$palette .= sprintf("%s c #%02x%02x%02x\n", chr($colidx++), $r, $g, $b );
	$cxx_init_code .= "\t\t++header_idx;\n".
	                  "\t\tassert( header_idx < 16 );\n".
	                  "\t\tm.insert( make_pair(header_idx, ict($r,$g,$b).convert<double,1>()) );\n";
}

#############################
#
# XPM sample of the scheme
#
############################

my $COLORWIDTH=100;
my $numcolors = $colidx - 65;
my $W = $COLORWIDTH * $numcolors;

open (FILE, '>cscheme_sample.xpm');
print FILE "! XPM2\n" ;
print FILE "$W 128 $numcolors 1\n";
print(FILE $palette);
for (my $i=0; $i<128; ++$i) {
	for (my $col=65; $col<$colidx; ++$col) {
		my $sym = chr($col);
		for (my $n=0; $n<$COLORWIDTH; ++$n) {
			$xpm .= $sym;
		}
	}
	$xpm .="\n";
}
print(FILE $xpm);
close (FILE);

#############################
#
# C++ code file
#
#############################

# Copy the license from old_colourscheme.inc.cc...
open(IN, 'old_colourscheme.inc.cc');
my $license = "";
for (my $i=0; $i<19; ++$i) {
	$license .= <IN>;
}
close(IN);

open(CS, '>colourscheme.inc.cc');
printf CS $template,$license,$cxx_free_form,$cxx_init_code,$cxx_bad_bitrate;
close(CS);
